/*%
   Java host software API of ZTEX SDK
   Copyright (C) 2009-2023 ZTEX GmbH.
   http://www.ztex.de
   
   This Source Code Form is subject to the terms of the Mozilla Public
   License, v. 2.0. If a copy of the MPL was not distributed with this file,
   You can obtain one at http://mozilla.org/MPL/2.0/.

   Alternatively, the contents of this file may be used under the terms
   of the GNU General Public License Version 3, as described below:

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
%*/

package ztex;

import org.usb4java.*;

/** 
  * Thrown if a device runs with no or the wrong firmware, i.e. if the ZTEX descriptor is not found or damaged. */
public class InvalidFirmwareException extends ZtexException {

/** 
 * Constructs an instance from the given device and error message.
 * @param msg The error message.
 * @param dev the object that generated the error (can be null).
 */
    public InvalidFirmwareException ( Object dev, String msg) {
	super( dev, "Invalid firmware: " + msg );
    } 

/** 
 * Constructs an instance from the given device and error message.
 * @param dev the object that generated the error (can be null).
 */
    public InvalidFirmwareException ( Object dev ) {
	super( dev, "Invalid firmware");
    } 
}    
