/*%
   Java host software API of ZTEX SDK
   Copyright (C) 2009-2023 ZTEX GmbH.
   http://www.ztex.de
   
   This Source Code Form is subject to the terms of the Mozilla Public
   License, v. 2.0. If a copy of the MPL was not distributed with this file,
   You can obtain one at http://mozilla.org/MPL/2.0/.

   Alternatively, the contents of this file may be used under the terms
   of the GNU General Public License Version 3, as described below:

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
%*/
package ztex;

import java.io.*;
import java.util.*;
import java.text.*;
import java.nio.*;

import ztex.*;

// *****************************************************************************
// ******* SimpleLogger ********************************************************
// *****************************************************************************
/** This class provides simple logging functionality.
  */
public class SimpleLogger {
    /** Log level.  <pre>
      * &gt;0: Log errors only 
      *  0: Log errors and messages
      * &lt;0: Log errors, messages and debug messages
      </pre>
      */    
    public static int logLevel = 0;

    /** Default logger for errors. If not overwritten, errors are printed to System.err
      */  
    public static SimpleLogger err = new SimpleLogger(1);
    /** Default logger for messages. If not overwritten, messages are printed to System.out
      */
    public static SimpleLogger log = new SimpleLogger(0);
    /** Default logger for debug messages. If not overwritten, debug messages are printed to System.out
      */
    public static SimpleLogger debugLog = new SimpleLogger(-1);

    private int ll;
    private String nl;

    /** Creates the default logger for a certain log level.
      * @param ll Log level for which the default logger is created
      */
    public SimpleLogger(int ll) {
	this.ll = ll;
	this.nl = System.getProperty("line.separator");
    }
    
    /** Writes a log message without finishing the line.
      * @param s The message to print
      */
    public void print(String s) {
	if ( (ll<0) && (logLevel<0) ) System.out.print(s);
	else if ( (ll==0) && (logLevel<1) ) System.out.print(s);
	else if ( ll>0 ) System.err.print(s);
    }

    /** Prints a new line. 
      * This is equal to invoking print(System.getProperty("line.separator"));
      */
    public void println() {
	print(nl);
	if ( ll>0 ) System.err.flush();
	else if (logLevel<1) System.out.flush();
    }

    /** Writes a log message and finishes the line.
      * This is equal to invoking print(s); and println();
      * @param s The message to print
      */
    public void println(String s) {
	print(s);
	println();
    }
}
